require 'securerandom'

require 'correlation_id/version'

require 'correlation_id/middleware/airbrake' if defined?(Airbrake)
require 'correlation_id/middleware/faraday' if defined?(Faraday)
require 'correlation_id/middleware/rack' if defined?(Rack)
require 'correlation_id/middleware/railtie' if defined?(Rails)

module CorrelationId
  HEADER_KEY = 'X-Correlation-ID'.freeze
  THREAD_KEY = 'CORRELATION_UUID'.freeze

  class << self
    def uuid
      Thread.current[THREAD_KEY] ||= _uuid
    end

    def uuid!
      Thread.current[THREAD_KEY] = _uuid
    end

    def uuid=(value)
      Thread.current[THREAD_KEY] = value
    end

    def disable
      using_uuid(_uuid) { yield }
    end

    def using_uuid(new_uuid)
      old_uuid = uuid

      self.uuid = new_uuid
      result = yield
      self.uuid = old_uuid

      result
    end

    private

    def _uuid
      SecureRandom.uuid
    end
  end
end
